#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <krb5.h>
#include "cgi.h"

#define CRH		"../html-ssl/crh.html"
#define KPASSWD_HTML    "../html-ssl/kpasswd.html"
#define NEW_HTML	"../html-ssl/set.html"
#define OOPS		"../html-ssl/oops.html"

char *err = NULL;
char *title = NULL;

struct cgi_list cl[] = {
#define CL_OLDPW	0
	{ "oldpw",	NULL },
#define CL_NEWPW	1
	{ "newpw",	NULL },
#define CL_NEWPW2	2
	{ "newpw2",	NULL },
#define CL_COMMAND	3
	{ "command",	NULL },
	{ NULL,		NULL },
};


    void
subfile( char *filename )
{
    FILE	*fs;
    int 	c;

    if (( fs = fopen( filename, "r" )) == NULL ) {
	perror( filename );
	exit( 1 );
    }

    fputs( "Content-type: text/html\n\n", stdout );

    while (( c = getc( fs )) != EOF ) {
	if ( c == '$' ) {

	    switch ( c = getc( fs )) {
	    case 't':
		if ( title != NULL ) {
		    printf( "%s", title );
		}
		break;

	    case 'e':
		if ( err != NULL ) {
		    printf( "%s", err );
		}
		break;

	    case 'o':
		if (( cl[ CL_OLDPW ].cl_data != NULL ) &&
			( *cl[ CL_OLDPW ].cl_data != '\0' )) {
		    printf( "%s", cl[ CL_OLDPW ].cl_data );
		}
		break;

	    case 's':
		printf( "%s", getenv("SCRIPT_NAME") );
		break;

	    case EOF:
		putchar( '$' );
		break;

	    case '$':
		putchar( c );
		break;

	    default:
		putchar( '$' );
		putchar( c );
	    }
	} else {
	    putchar( c );
	}
    }
    if ( fclose( fs ) != 0 ) {
	perror( filename );
    }

    return;
}


    int
main()
{
    krb5_error_code         kerror;
    krb5_context            kcontext;
    krb5_principal          kprinc;
    krb5_get_init_creds_opt kopts;
    krb5_creds              kcreds;
    int                     error_code;
    krb5_data               kd_rcs, kd_rs;
    char		    *tmpl = KPASSWD_HTML;
    char		    *trim;

    if ( cgi_info( CGI_STDIN, cl ) != 0 ) {
	fprintf( stderr, "%s: cgi_info broken\n", getenv( "SCRIPT_NAME" ) );
	exit( 1 );
    }

    if (( cl[ CL_COMMAND ].cl_data != NULL ) &&
	    ( *cl[ CL_COMMAND ].cl_data != '\0' ) &&
	    ( cl[ CL_COMMAND ].cl_data == "set" )) {
	tmpl = NEW_HTML;
    }

    if (( cl[ CL_OLDPW ].cl_data == NULL ) ||
	    ( *cl[ CL_OLDPW ].cl_data  == '\0' ) ||
	    ( cl[ CL_NEWPW ].cl_data   == NULL ) ||
	    ( *cl[ CL_NEWPW ].cl_data  == '\0' ) ||
	    ( cl[ CL_NEWPW2 ].cl_data  == NULL ) ||
	    ( *cl[ CL_NEWPW2 ].cl_data == '\0' )) {
	err = "All fields are required.";
	title = "Error";
	goto dispage;
    }

    if (( kerror = krb5_init_context( &kcontext ))) {
	err = error_message( kerror );
	tmpl = OOPS;
	goto dispage;
    }

    if (( kerror = krb5_parse_name( kcontext, getenv( "REMOTE_USER" ),
	    &kprinc ))) {
	err = error_message( kerror );
	tmpl = OOPS;
	goto dispage;
    }

    krb5_get_init_creds_opt_init( &kopts );
    krb5_get_init_creds_opt_set_tkt_life( &kopts, 5*60 );
    krb5_get_init_creds_opt_set_renew_life( &kopts, 0 );
    krb5_get_init_creds_opt_set_forwardable( &kopts, 0 );
    krb5_get_init_creds_opt_set_proxiable( &kopts, 0 );

    if (( kerror = krb5_get_init_creds_password( kcontext, &kcreds, 
	    kprinc, cl[ CL_OLDPW ].cl_data, krb5_prompter_posix, NULL, 0, 
	    "kadmin/changepw", &kopts ))) {
	if ( kerror == KRB5KRB_AP_ERR_BAD_INTEGRITY ) {
	    cl[ CL_OLDPW ].cl_data = NULL;

	    err = "Old password incorrect.  Is [caps lock] on?";
	    title = "Password Incorrect";
	    goto dispage;
	} else {
	    err = error_message( kerror );
	    goto dispage;
	}
    }

    if ( strcmp( cl[ CL_NEWPW ].cl_data, cl[ CL_NEWPW2 ].cl_data )) {
	err = "Your new passwords do not match.";
	title = "Password Mismatch";
	goto dispage;
    }

    if (( kerror = krb5_change_password( kcontext, &kcreds, 
	    cl[ CL_NEWPW ].cl_data, &error_code, &kd_rcs, &kd_rs ))) {
	err = error_message( kerror );
	tmpl = OOPS;
	goto dispage;
    }

    if ( error_code != 0 ) {
	kd_rs.data[ kd_rs.length ] = '\0';
	kd_rcs.data[ kd_rcs.length ] = '\0';

	/* only the first line of text returned doesn't suck ass */
	if (( err = strtok( kd_rs.data, "\n" )) != NULL ) {
	    err = kd_rs.data;
	}

	if (( trim = strchr( kd_rs.data, ':' )) != NULL ) {
	    err = trim;
	    err++;
	}

	title = kd_rcs.data;
	goto dispage;
    }

    krb5_free_data_contents( kcontext, &kd_rs );
    krb5_free_data_contents( kcontext, &kd_rcs );

    err = "Your password has been set.";
    tmpl = CRH;

dispage:
    subfile ( tmpl );

    exit( 0 );
}
